@@ -11,22 +11,22 @@ from typing import Dict
 from typing import Tuple
 
 import inflection
+import yaml
+import glob
 
 from ads.common.auth import get_signer
 from ads.opctl import logger
 from ads.opctl.config.base import ConfigProcessor
-from ads.opctl.config.utils import NotSupportedError
+from ads.opctl.config.utils import NotSupportedError, convert_notebook
 from ads.opctl.constants import (
     ML_JOB_GPU_IMAGE,
     ML_JOB_IMAGE,
-    OPS_IMAGE_BASE,
-    OPS_IMAGE_GPU_BASE,
 )
 from ads.opctl.utils import (
-    _list_ads_operators,
-    _parse_conda_uri,
-    _get_region_key,
-    _get_namespace,
+    list_ads_operators,
+    parse_conda_uri,
+    get_region_key,
+    get_namespace,
 )
 
 
@@ -51,31 +51,27 @@ class ConfigResolver(ConfigProcessor):
 
     def __init__(self, config: Dict = None) -> None:
         super().__init__(config)
-        self.ads_operators = _list_ads_operators()
+        self.ads_operators = list_ads_operators()
 
     def process(self):
         # this function should be run after merging configs
         # conda pack scenarios --
-        # - user runs their own scripts and commands -> source_folder + entry script + (command)
+        # - user runs their own scripts and commands -> source_folder + entrypoint + (command)
         # - user runs ADS operator -> name/YAML
-        # - user runs custom operator -> source_folder + YAML
-        # docker image scenarios --
-        # - user runs their own docker image -> image (to run) + (entry script) + (command)
-        # - user runs ADS operator -> name/YAML
-        # - user runs custom operator -> (image if remote) + source folder + YAML
 
-        if (
-            self.config["execution"].get("job_id")
-            and self.config["execution"].get("backend", None) == "job"
-        ):
+        # docker image scenarios --
+        # - user runs their own docker image -> image (to run) + (entrypoint) + (command)
+        # - user runs ADS operator -> name/YAML
+
+        if self.config["execution"].get("job_id"):
             return self
 
         self._resolve_operator_name()
 
         if not (
             self._is_ads_operator()
-            or self.config["execution"].get("source_folder", None)
-            or self.config["execution"].get("image", None)
+            or self.config["execution"].get("source_folder")
+            or self.config["execution"].get("image")
         ):
             raise ValueError(
                 "Either an ADS operator or a source folder or a docker image needs to be given."
@@ -83,52 +79,45 @@ class ConfigResolver(ConfigProcessor):
 
         self._resolve_conda()
         if not (
-            self._is_operator()
-            or self.config["execution"].get("conda_slug", None)
-            or self.config["execution"].get("image", None)
+            self.config["execution"].get("conda_slug")
+            or self.config["execution"].get("image")
+            or self.config["execution"]["backend"] == "dataflow"
         ):
             raise ValueError(
                 "If not running an operator, conda pack info or image name needs to be given."
             )
         if self.config["execution"].get("conda_slug") and self.config["execution"].get(
-            "image", None
+            "image"
         ):
             raise ValueError(
                 "Both conda pack info and image name are given. It is ambiguous which one to use."
             )
 
         self._resolve_source_folder_path()
+        self._resolve_entry_script()
         self._resolve_command()
         self._resolve_image_name()
         self._resolve_env_vars()
-        self._reslve_mounted_volumes()
+        self._resolve_mounted_volumes()
         self._resolve_job_name()
 
         logger.debug(f"Config: {self.config}")
 
         return self
 
-    def _is_operator(self) -> bool:
-        return (
-            self.config["execution"].get("operator_name", None)
-            or self.config.get("kind", None) == "MLOperator"
-        )
-
     def _is_ads_operator(self) -> bool:
-        return self.config["execution"].get("operator_name", None) in self.ads_operators
+        return self.config["execution"].get("operator_name") in self.ads_operators
 
-    def _resolve_operator_name(self) -> str:
-        exec_config = self.config["execution"]
-        operator_name = exec_config.get("operator_name", None) or self.config.get(
-            "name", None
-        )
-        self.config["execution"]["operator_name"] = operator_name
+    def _resolve_operator_name(self) -> None:
+        self.config["execution"]["operator_name"] = self.config["execution"].get(
+            "operator_name"
+        ) or self.config.get("name")
 
-    def _resolve_source_folder_path(self) -> str:
+    def _resolve_source_folder_path(self) -> None:
         # this should be run after resolve_operator_name()
         # resolve ADS operator source folder path
         exec_config = self.config["execution"]
-        if exec_config.get("source_folder", None):
+        if exec_config.get("source_folder"):
             self.config["execution"]["source_folder"] = os.path.abspath(
                 os.path.normpath(exec_config["source_folder"])
             )
@@ -152,23 +141,31 @@ class ConfigResolver(ConfigProcessor):
             else:
                 self.config["execution"]["source_folder"] = None
 
-    def _resolve_command(self) -> str:
-        # this should be run after resolve_operator_name and resolve_conda
-        # resolve command for operators (ADS or custom)
-        exec_config = self.config["execution"]
-        if exec_config.get("entrypoint", None) and exec_config.get("conda_slug", None):
-            # do not touch in case of running a user script with conda pack
-            self.config["execution"]["command"] = exec_config.get("cmd_args", None)
-        elif not self._is_operator() and exec_config.get("image", None):
-            # do not touch case of running arbitrary image
-            self.config["execution"]["command"] = exec_config.get("command", None)
-        else:
+    def _resolve_entry_script(self) -> None:
+        # this should be run after _resolve_source_folder_path
+        if not self._is_ads_operator():
+            if os.path.splitext(self.config["execution"]["entrypoint"])[1] == ".ipynb":
+                input_path = os.path.join(
+                    self.config["execution"]["source_folder"],
+                    self.config["execution"]["entrypoint"],
+                )
+                exclude_tags = self.config["execution"].get("exclude_tag")
+                self.config["execution"]["entrypoint"] = os.path.basename(
+                    convert_notebook(
+                        input_path,
+                        {},
+                        exclude_tags,
+                        overwrite=self.config["execution"].get("overwrite", False),
+                    )
+                )
 
-            # the rest scenarios are running operators
+    def _resolve_command(self) -> None:
+        # this should be run after resolve_operator_name and resolve_conda
+        # resolve command for operators
+        exec_config = self.config["execution"]
+        if self._is_ads_operator():
             logger.info("Building command for operator runs...")
-            operator_name = exec_config.get("operator_name", None) or os.path.basename(
-                exec_config["source_folder"]
-            )
+            operator_name = exec_config.get("operator_name")
             command = f"-n {operator_name} -c {exec_config['oci_config']} -p {exec_config['oci_profile']}"
             if "spec" in self.config:
                 encoded_spec = base64.b64encode(
@@ -176,8 +173,13 @@ class ConfigResolver(ConfigProcessor):
                 ).decode()
                 command += f" -s {encoded_spec}"
             self.config["execution"]["command"] = command
+        else:
+            # cmd_args is for running scripts, either locally, with jobs or dataflow; command is for docker command
+            self.config["execution"]["command"] = exec_config.get(
+                "cmd_args"
+            ) or exec_config.get("command")
 
-    def _resolve_conda(self) -> Tuple[str, str]:
+    def _resolve_conda(self) -> None:
         # this should be run after resolve_operator_name
         exec_config = self.config["execution"]
         # two scenarios that indicate conda pack should be used:
@@ -185,16 +187,20 @@ class ConfigResolver(ConfigProcessor):
         # - use-conda is turned on explicitly. in this case for ads operators, look up index.yaml to fill in conda info
         slug = None
         conda_type = None
-        if exec_config.get("conda_uri", None):
-            slug = _parse_conda_uri(exec_config["conda_uri"])[-1]
+        if exec_config.get("conda_uri"):
+            slug = parse_conda_uri(exec_config["conda_uri"])[-1]
             conda_type = "published"
-        elif exec_config.get("conda_slug", None):
+        elif exec_config.get("conda_slug"):
             slug = exec_config["conda_slug"]
-            conda_type = "service"
+            conda_uri, conda_type = self._get_conda_uri_and_type(slug)
+            self.config["execution"]["conda_uri"] = conda_uri
+            if conda_type and conda_type == "data_science":
+                conda_type = "service"
+            conda_type = conda_type or "service"
         elif exec_config.get("use_conda", False):
-            operator_name = exec_config.get("operator_name", None)
+            operator_name = exec_config.get("operator_name")
             if self._is_ads_operator():
-                if self.ads_operators[operator_name].get("conda_slug", None):
+                if self.ads_operators[operator_name].get("conda_slug"):
                     slug = self.ads_operators[operator_name]["conda_slug"]
                     conda_type = "service"
                 else:
@@ -206,7 +212,21 @@ class ConfigResolver(ConfigProcessor):
         self.config["execution"]["conda_slug"] = slug
         self.config["execution"]["conda_type"] = conda_type
 
-    def _resolve_image_name(self) -> str:
+    def _get_conda_uri_and_type(self, slug: str) -> Tuple[str, str]:
+        conda_pack_path = os.path.join(
+            os.path.expanduser(self.config["execution"]["conda_pack_folder"]), slug
+        )
+        if os.path.exists(conda_pack_path):
+            manifest_path = glob.glob(os.path.join(conda_pack_path, "*_manifest.yaml"))[
+                0
+            ]
+            with open(manifest_path) as f:
+                manifest = yaml.safe_load(f.read())["manifest"]
+            return manifest.get("pack_uri"), manifest.get("type")
+        else:
+            return None, None
+
+    def _resolve_image_name(self) -> None:
         # this should be run after resolve_conda() and resolve_command()
         exec_config = self.config["execution"]
         if exec_config.get("conda_slug", None):
@@ -214,17 +234,17 @@ class ConfigResolver(ConfigProcessor):
                 ML_JOB_GPU_IMAGE if exec_config.get("gpu", False) else ML_JOB_IMAGE
             )
         elif self._is_ads_operator():
-            image = self.ads_operators[exec_config["operator_name"]].get("image", None)
+            image = self.ads_operators[exec_config["operator_name"]].get("image")
             oci_auth = get_signer(
-                self.config["execution"].get("oci_config", None),
-                self.config["execution"].get("oci_profile", None),
+                self.config["execution"].get("oci_config"),
+                self.config["execution"].get("oci_profile"),
             )
-            region_key = _get_region_key(oci_auth).lower()
-            if exec_config.get("namespace", None):
+            region_key = get_region_key(oci_auth).lower()
+            if exec_config.get("namespace"):
                 namespace = exec_config["namespace"]
             else:
-                namespace = _get_namespace(oci_auth)
-            if self.config.get("infrastructure", {}).get("docker_registry", None):
+                namespace = get_namespace(oci_auth)
+            if self.config.get("infrastructure", {}).get("docker_registry"):
                 self.config["execution"]["image"] = os.path.join(
                     self.config["infrastructure"]["docker_registry"], image
                 )
@@ -232,17 +252,10 @@ class ConfigResolver(ConfigProcessor):
                 self.config["execution"][
                     "image"
                 ] = f"{region_key}.ocir.io/{namespace}/{image}"
-        elif (
-            self._is_operator()
-            and self.config["execution"].get("backend", None) == "local"
-        ):
-            self.config["execution"]["image"] = (
-                OPS_IMAGE_GPU_BASE if exec_config.get("gpu", False) else OPS_IMAGE_BASE
-            )
 
-    def _resolve_env_vars(self) -> dict:
+    def _resolve_env_vars(self) -> None:
         env_vars = self.config["execution"].get("env_vars", {})
-        if self.config["execution"].get("env_var", None):
+        if self.config["execution"].get("env_var"):
             for ev in self.config["execution"]["env_var"]:
                 if "=" in ev:
                     k, v = ev.split("=")
@@ -256,9 +269,9 @@ class ConfigResolver(ConfigProcessor):
                     )
         self.config["execution"]["env_vars"] = env_vars
 
-    def _reslve_mounted_volumes(self) -> dict:
+    def _resolve_mounted_volumes(self) -> None:
         volumes = self.config["execution"].get("volumes", {})
-        if self.config["execution"].get("volume", None):
+        if self.config["execution"].get("volume"):
             for v in self.config["execution"]["volume"]:
                 host, remote = v.split(":")
                 if not os.path.exists(host.strip()):
@@ -266,10 +279,10 @@ class ConfigResolver(ConfigProcessor):
                 volumes[os.path.abspath(host.strip())] = {"bind": remote.strip()}
         self.config["execution"]["volumes"] = volumes
 
-    def _resolve_job_name(self) -> str:
+    def _resolve_job_name(self) -> None:
         # this should be run after resolve_operator_name
-        if not self.config["execution"].get("job_name", None):
-            if self.config["execution"].get("operator_name", None):
+        if not self.config["execution"].get("job_name"):
+            if self.config["execution"].get("operator_name"):
                 self.config["execution"]["job_name"] = self.config["execution"][
                     "operator_name"
                 ]
