@@ -19,3 +19,7 @@ ADS_JOBS_CONFIG_FILE_NAME = "ml_job_config.ini"
 ADS_DATAFLOW_CONFIG_FILE_NAME = "dataflow_config.ini"
 DEFAULT_IMAGE_HOME_DIR = "/home/datascience"
 DEFAULT_IMAGE_SCRIPT_DIR = "/etc/datascience"
+DEFAULT_IMAGE_CONDA_DIR = "/opt/conda/envs"
+DEFAULT_NOTEBOOK_SESSION_SPARK_CONF_DIR = "/home/datascience/spark_conf_dir"
+DEFAULT_NOTEBOOK_SESSION_CONDA_DIR = "/home/datascience/conda"
+ADS_DATAFLOW_CONFIG_FILE_NAME = "dataflow_config.ini"
